local yard=TRAININGGROUNDS_Mobile_ConvertYardsToTGUnits(1)
local CIRCLE_RADIUS=yard*7.5

-- This is the aura attached to the void zone(s).
do
	local super=TRAININGGROUNDS_Aura_VoidZone
	TRAININGGROUNDS_JAINA_Aura_ArcticGround=TRAININGGROUNDS_inheritsFrom(super)
	local class=TRAININGGROUNDS_JAINA_Aura_ArcticGround
	
	function class:TickEffect(percentage)
		super.TickEffect(self,percentage)
		local gdm=self.targetcombat.owner.groupdancemodule
		--gdm:ReactToCustomDanceEvent({action=TRAININGGROUNDS_JAINA_CustomDanceEventType.ARCTIC_GROUND,
			--source=player.combat})		
		--TODO: why is gdm sometimes nil here?  prismatic image?
		if(gdm)then
			if(self.targetcombat.owner==gdm.player)then
				gdm:ApplyChillingTouchStacks(1,false)
			end
		end
	end
	
end

-- This is the "real" void zone.  It polls the fake void zones for collision data.
do
	local super=TRAININGGROUNDS_VoidZone
	TRAININGGROUNDS_JAINA_VoidZone_ArcticGround = TRAININGGROUNDS_inheritsFrom(super)
	local class=TRAININGGROUNDS_JAINA_VoidZone_ArcticGround 
	
	function class:Setup(environment,caster)
		super.Setup(self,environment,caster)
		caster.voidzones=self
	end
	
	function class:SetCustomInfo()
		super.SetCustomInfo(self)
		
		self.aura=TRAININGGROUNDS_JAINA_Aura_ArcticGround
		
		-- Cone of Death
		self.cones={}
		-- Soulblight
		self.circles={}
		
		self:CalculateVoidColor()
	end
	
	function class:Step(elapsed)		
		self:CalculateVoidColor()

		for i=#self.cones,1,-1 do
			if(self.cones[i].dead)then
				tremove(self.cones,i)
			end
		end
		for i=#self.circles,1,-1 do
			if(self.circles[i].dead)then
				tremove(self.circles,i)
			end		
		end
				
	end
	
	function class:AddCone(cone)
		tinsert(self.cones,cone)
	end
	
	function class:AddCircle(circle)
		tinsert(self.circles,circle)
	end	
	
	function class:CollisionCheck(mobile)
		--TODO: possibly line-triangle collision instead
		local hit=false
		--print("#Cones:",#self.cones)
		for i=1,#self.cones do
			local cone=self.cones[i]		
			hit=TRAININGGROUNDS_PointTriangleCollision(mobile.x,mobile.y,cone.x1,cone.y1,cone.x2,cone.y2,cone.x3,cone.y3)
			if(hit)then
				break
			end
		end
		-- --if we hit a cone, we can skip over the circle check
		if(not hit)then
			for i=1,#self.circles do
				local circle=self.circles[i]
				-- Arctic Ground circles do NOT have a graceperiod (do they?)
				hit=TRAININGGROUNDS_PointCircleCollision(mobile.x,mobile.y,circle.x,circle.y,circle.width/2)				
				if(hit)then
					break
				end
			end
		end
		--we checked both cones and circles; now act on it
		if(hit)then	
			self:OnEnterVoidZone(mobile)
		else
			self:OnLeaveVoidZone(mobile)
		end
	end
	
	function class:CalculateVoidColor()
		local t=self.environment.localtime*0.5
		local t2=t*1.5
		-- local brightness=0.75+0.25*math.cos(t2)
		-- self.voidcolorR=0--(0.125+0.125)*math.cos(t)*brightness
		-- self.voidcolorG=0
		-- self.voidcolorB=0.25*brightness
		self.voidcolorR=0.75
		self.voidcolorG=0.75
		self.voidcolorB=1		
		self.voidcolorA=1.0
	end
	
	function class:SetVoidColor(voidobject)
		if(not self.voidcolorR)then
			self:CalculateVoidColor()
		end
	end

end



-- This is a "fake" void zone.
--TODO: lots of overlap with Telegraph_Cone here
do
	local super=TRAININGGROUNDS_GameObject
	TRAININGGROUNDS_JAINA_FakeVoidZone_ArcticGround_Cone = TRAININGGROUNDS_inheritsFrom(super)
	local class=TRAININGGROUNDS_JAINA_FakeVoidZone_ArcticGround_Cone

	function class:Setup(environment,spellinstance)
		super.Setup(self,environment)
		if(not spellinstance.castercombat.voidzones)then
			local voidzones=TRAININGGROUNDS_JAINA_VoidZone_ArcticGround.new()			
			voidzones:Setup(environment,spellinstance.castercombat)	
		end				
		spellinstance.castercombat.voidzones:AddCone(self)
		self.voidzonecontroller=spellinstance.castercombat.voidzones
		self.spawntime=environment.localtime
		self.graceperiod=0
		self.expirytime=environment.localtime+20
	end
	
	function class:SetCustomInfo()
		super.SetCustomInfo(self)
		self.x1=0;self.y1=0;self.x2=0;self.y2=0;self.x3=0;self.y3=0
	end
		
	function class:SetDrawable()
		self.drawable=TRAININGGROUNDS_Drawable_Polygon_VoidZone.new()		
		self.drawable:Setup(self,self.environment)
	end
	
	function class:PreDraw()
		super.PreDraw(self)
		local r,g,b,a=self.voidzonecontroller.voidcolorR,self.voidzonecontroller.voidcolorG,self.voidzonecontroller.voidcolorB,self.voidzonecontroller.voidcolorA
		local timeelapsedsincespawn=self.environment.localtime-self.spawntime
		local flash=1*(2*(0.5-timeelapsedsincespawn))
		if(flash>1)then flash=1 end
		if(flash<0)then flash=0 end
		r=r+((1-r)*flash)
		g=g+((1-g)*flash)
		b=b+((1-b)*flash)
		local expirytimeremaining=self.expirytime-self.environment.localtime
		if(expirytimeremaining>1)then expirytimeremaining=1 end
		self.drawable:SetColor(r,g,b,a*expirytimeremaining)
	end
	
	function class:SetVertices(x1,y1,x2,y2,x3,y3)
		self.x1=x1;self.y1=y1;self.x2=x2;self.y2=y2;self.x3=x3;self.y3=y3
		self.drawable:SetVertices(x1,y1,x2,y2,x3,y3)		
	end
end



--this is another "fake" void zone.
do
	local super=TRAININGGROUNDS_GameObject
	TRAININGGROUNDS_JAINA_FakeVoidZone_ArcticGround_Circle=TRAININGGROUNDS_inheritsFrom(super)
	local class=TRAININGGROUNDS_JAINA_FakeVoidZone_ArcticGround_Circle

	function class:Setup(environment,spellinstance)
		super.Setup(self,environment)
		if(not spellinstance.castercombat.voidzones)then
			local voidzones=TRAININGGROUNDS_JAINA_VoidZone_ArcticGround.new()
			voidzones:Setup(environment,spellinstance.castercombat)	
		end			
		spellinstance.castercombat.voidzones:AddCircle(self)
		self.voidzonecontroller=spellinstance.castercombat.voidzones
		self.spawntime=environment.localtime
		self.graceperiod=0
		self.expirytime=environment.localtime+20
	end
	
	
	function class:SetCustomInfo()
		super.SetCustomInfo(self)
		--TODO: SetGracePeriodInTicks(1)
		--TODO: does the grace period apply to the circle itself, or to the player who dropped the circle?
		self.width=CIRCLE_RADIUS*2
		self.height=self.width
	end
		
	function class:SetDrawable()
		self.drawable=TRAININGGROUNDS_Drawable_Sprite_Circle_VoidZone.new()		
		self.drawable:Setup(self,self.environment)
		self.drawable.width=self.width
		self.drawable.height=self.height		
	end
	
	function class:PreDraw()
		super.PreDraw(self)
		local r,g,b,a=self.voidzonecontroller.voidcolorR,self.voidzonecontroller.voidcolorG,self.voidzonecontroller.voidcolorB,self.voidzonecontroller.voidcolorA
		local timeelapsedsincespawn=self.environment.localtime-self.spawntime
		local flash=1*(2*(0.5-timeelapsedsincespawn))
		if(flash>1)then flash=1 end
		if(flash<0)then flash=0 end
		r=r+((1-r)*flash)
		g=g+((1-g)*flash)
		b=b+((1-b)*flash)
		local expirytimeremaining=self.expirytime-self.environment.localtime
		if(expirytimeremaining>1)then expirytimeremaining=1 end
		self.drawable:SetColor(r,g,b,a*expirytimeremaining)
	end



end