
-- We currently support 0-1 Drawables per GameObject.
-- 

-- NEW: drawables may have more than one wow frame.
-- All drawables currently MUST have a self.frame (i.e. that frame must not be nil)
-- but if self.frames exists, functions will operate on that INSTEAD OF self.frame.
	-- (so make sure self.frame is added to self.frames if it's important)
	
--TODO: check if ysortable is set.  if it is, throw an error.  (ysortable is supposed to be set in drawable's owner instead)

TRAININGGROUNDS_Drawable = {}
TRAININGGROUNDS_Drawable.__index = TRAININGGROUNDS_Drawable

function TRAININGGROUNDS_Drawable.new()
	local self=setmetatable({}, TRAININGGROUNDS_Drawable)	
	
	return self
end

function TRAININGGROUNDS_Drawable:Setup(owner,environment)
	self.owner=owner
	self.environment=environment
	self:SetCustomInfo()
	self.frames=nil
	self:CreateFrame()
	
	self.parentframe=self.environment.frame
	
	--self.frame:SetParent(environment.frame)	
	--self.frame:SetParent(environment.scrollframe)	
	self.x=0
	self.y=0
	self.z=0
	self.alpha=1
	
	self.frame:Show()
	self.frame.owner=self
	self.frame.texture:Show() --necessary in order to respect scrollframe
	self.frame.texture:SetColorTexture(0,0,0,0)	--SetAlpha apparently doesn't work here, but SetColorTexture does
	self:SetFrameAppearance()
		
		
	self:SetFrameLayer()
	self:SetFrameLevel()
	
	self:SetClickability()
	
	self:SetAnimationSystem()

	-- SetFrameLevel doesn't take effect on the frame the object is created
		-- so we attempt to set it on the 2nd frame.
	self.updateframelevel=true
	
	
end



function TRAININGGROUNDS_Drawable:SetCustomInfo()
	self.anchor="BOTTOM"
	self.width=100
	self.height=100	
end

function TRAININGGROUNDS_Drawable:CreateFrame()
	--override; don't call super while overriding
	self.frame=TRAININGGROUNDS_ReusableFrames:GetFrame()
	--self.frame.texture:Show()
end

function TRAININGGROUNDS_Drawable:SetFrameAppearance()
	--override; don't call super while overriding
end

function TRAININGGROUNDS_Drawable:SetFrameLayer()
	--override; don't call super while overriding
	self.framelayer=self.environment.cameramanager.defaultframelayer
	--print("Framelayer",self.framelayer)
end

function TRAININGGROUNDS_Drawable:SetFrameLevel()
	--override; don't call super while overriding
	-- if(self.framelayer.level==255)then
		
	-- end
	-- print("Framelevel ", self, self.framelayer.level)
	
	
	
	
	-- if(self.visible and self.framelayer==self.environment.cameramanager.camera.defaultframelayer)then
		-- --TODO: instead of checking self.visible, it would be better simply not to set a drawable in the first place...
		-- print("Drawable's framelayer was not set.  Attempting to crash trace...")		
		-- local crashobj=self.owner:Class().new()
		-- -- TODO: some objs don't even have a SetCustomInfo.  we may need a "crashtrace=function() error(0) end" addition to the class template
		-- crashobj.SetCustomInfo(nil)
	-- end
	--print("framelayer.level:",self.framelayer,self.framelayer.level)
	
	if(self.frames)then
		for i=1,#self.frames do
			self.frames[i]:SetFrameLevel(self.framelayer.level)
		end
	else
		self.frame:SetFrameLevel(self.framelayer.level)
	end
	--self.frame:SetFrameLevel(self.framelayer.level)
end


function TRAININGGROUNDS_Drawable:SetClickability()
	-- override. don't call super when overriding
	-- setup function will create a debug-visible clickframe if clickable==true AND frame is visible and not nil.
	-- for various convenience reasons, clickarea is setpoint(bottom, bottom) instead of center/center or bottomleft etc.
	--TODO LATER: maybe option for center+center?
		
	--TODO: clickarea should be defined in drawable and scale up with framescale
	
	-- remember to call SetClickability again if you ever resize an object after setup!
	
	self.clickable=false	
	self.clickarea={}
	self.clickarea.x=0*4; self.clickarea.y=0*4; self.clickarea.width=100; self.clickarea.height=100	
end

function TRAININGGROUNDS_Drawable:SetAnimationSystem()
	--TODO: change to empty virtual function
	-- animation is not used by base drawable class, but can be used by child classes	
	self.animationsystem=TRAININGGROUNDS_AnimationSystem.new()
	self.animationsystem:Setup(self)
end

function TRAININGGROUNDS_Drawable:SetAlpha(alpha)
	self.alpha=alpha
end

function TRAININGGROUNDS_Drawable:ShowFrames()	
	if(self.frames)then
		for i=1,#self.frames do
			self.frames[i]:SetAlpha(self.alpha)
		end
	else
		--print("ShowFrames() alpha",self.alpha)
		--TRAININGGROUNDS_FrameVisibilityCheck(self.frame)
		self.frame:SetAlpha(self.alpha)
	end
end

function TRAININGGROUNDS_Drawable:HideFrames()	
	if(self.frames)then
		for i=1,#self.frames do
			self.frames[i]:SetAlpha(0)
		end
	else
		self.frame:SetAlpha(0)
	end
end


function TRAININGGROUNDS_Drawable:SetEquipment(equipment)
	--override
	--TODO: maybe throw a warning message if we call SetEquipment on a drawable that doesn't support it
end

-- mostly used for ysort...
function TRAININGGROUNDS_Drawable:SetSortedFrameLevels(level)
	if(self.frames)then
		for i=1,#self.frames do
			self.frames[i].framelevel=level
			self.frames[i]:SetFrameLevel(level)
		end
	else
		self.frame.framelevel=level
		self.frame:SetFrameLevel(level)
	end
	
	----TODO: modelframe might still be needed, but what was "nameplate" used for?  resources?
	-- if(v.modelframe~=nil)then 
		-- v.modelframe.framelevel=layer
		-- v.modelframe:SetFrameLevel(layer) 					
	-- end	
	-- if(v.nameplate~=nil)then
		-- layer=layer+1
		-- v.nameplate:SetAllFrameLevels(layer)
	-- end
	
	level=level+1
	return level
end

-- --TODO: could probably use a rename for contrast with "sortedframelevels"
-- --TODO: use for shadows too (currently just resourcebars)
-- function TRAININGGROUNDS_Drawable:SetCustomFrameLevels(preysortlevel,postysortlevel)
	-- --override
-- end

-- called pretty much exclusively from 2d camera.
-- we must set frame size at the same time as orientation because model frames must use SetCameraPosition to set both.
-- yaw corresponds to model facing
-- pitch corresponds to camera height

-- TODO: WARNING: this function only operates on self.frame, not on self.frames table.  should that change?
		-- (we might not want it to change; the function args don't make sense when applied to multiple frames)
			-- (other frames in self.frames should be anchored to self.frame)
function TRAININGGROUNDS_Drawable:ArrangeFrame(parentanchor,x,y,wid,hgt,yaw,pitch,zoom)
	--override; don't call super when overriding
	self.frame:SetParent(self.parentframe)
	--print("Drawing",self.frame.name)
	-- if(self.frame.name=="TRAININGGROUNDS_ReusableFrame62")then
		-- print("Frame62's parent is",self.parentframe,self.frame:GetParent())
	-- end
	if(self.parentframe==UIParent)then		
		print("WARNING:",self.frame.name,"parent is UIParent (self.parentframe)")
	end
	if(self.frame:GetParent()==UIParent)then
		print("WARNING:",self.frame.name,"parent is UIParent (self.frame:GetParent)")
	end
	self.frame:ClearAllPoints()
	self.frame:SetPoint(self.anchor,self.parentframe,parentanchor,x,y)	
	self:SetFrameSizeAndOrientation(wid,hgt,yaw,pitch,zoom)
end


function TRAININGGROUNDS_Drawable:ArrangeFrame3D(cposoffsetvectorx,cposoffsetvectory,cposoffsetvectorz,cdist,clookatx,clookaty,clookatz,cangx,cangy,cangz)
	--camera angle parameters are unused here -- cpos offset vector is taken care of already
	-- they are used in child functions, such as nameplates
	
	--override; don't call super when overriding
	self.frame:SetParent(self.parentframe)
	--print("Drawing",self.frame.name)
	-- if(self.frame.name=="TRAININGGROUNDS_ReusableFrame62")then
		-- print("Frame62's parent is",self.parentframe,self.frame:GetParent())
	-- end
	if(self.parentframe==UIParent)then		
		print("WARNING:",self.frame.name,"parent is UIParent")
	end
	if(self.frame:GetParent()==UIParent)then
		print("WARNING:",self.frame.name,"parent is UIParent")
	end
	self.frame:ClearAllPoints()
	self.frame:SetAllPoints(self.parentframe)
	self.frame.texture:Show()
	
	
	--self:SetFrameSizeAndOrientation(wid,hgt,yaw,pitch,zoom)
	if(not self.frame.HasCustomCamera) then
		error("Assertion failed: tried to call ArrangeFrame3D on a non-ModelCamera frame",self.frame,self.frame.name,self.owner.name)	
	end
	if(not self.frame:HasCustomCamera())then
		--TODO: this might feasibly break reusableframes in some way
		self.frame:MakeCurrentCameraCustom()
		--TODO: assert() camera is now custom
	end

	--	* TODO: is this actually correct or are we changing modelfacing somewhere in PlayerCharacter or Drawable
		--TODO NEXT: run SetFacing(0) here
	-- (when cangy==0, facing "right"):
	-- positive "X": into screen, away from viewer
	-- positive "Y": left
	-- positive "Z": up
	
	

	
	--!!!	
	--print("ARRANGE",cposx,cposy,cposz,clookatx,clookaty,clookatz)
	--TODO: REMINDER: camera Y is height, object's Z is height

	local scale=1
	--TODO: figure out how we want to handle "scale" property
	if(self.scale3D)then scale=self.scale3D end	--TODO: move scale to modelinfo
	self.frame:SetScale(1)	
	self.frame:SetModelScale(scale)
	
	--local cameradriftmultiplier=1/(1.27*1.33)	--M pandaren, fullscreen
	local cameradriftmultiplier=TRAININGGROUNDS_Config.Camera.CameraDriftMultiplier
	if(self.camerapositionmultiplier)then cameradriftmultiplier=cameradriftmultiplier*self.camerapositionmultiplier end
	--cameradriftmultiplier=0--!!!

	
	clookatx=clookatx*cameradriftmultiplier
	clookaty=clookaty*cameradriftmultiplier
	clookatz=clookatz*cameradriftmultiplier
	-- translate camera to focus
	local cposx=cposoffsetvectorx*cdist+clookatx
	local cposy=cposoffsetvectory*cdist+clookaty
	local cposz=cposoffsetvectorz*cdist+clookatz
	
	 self.frame:SetCameraPosition(cposx,cposz,cposy)
	 self.frame:SetCameraTarget(clookatx,clookatz,clookaty)
	 


	

	--self.frame:SetScale(scale)
	--self.frame:SetPosition(0,2,0)

	local inverseyard=1/TRAININGGROUNDS_Mobile_ConvertYardsToTGUnits(1)
	--local positionmultiplier=1	--TODO: self.positionmultiplier (or get rid of positionmultiplier, maybe we can't use it)
	--local positionmultiplier=1.27*1.33	--M pandaren, fullscreen
	local positionmultiplier=1
	if(self.positionmultiplier)then positionmultiplier=self.positionmultiplier end
	

	local finalmultiplier=positionmultiplier*inverseyard
	--local finalmultiplier=positionmultiplier*inverseyard/scale
	

	local adjustedx,adjustedy,adjustedz=self:Get3DPositionCoords()
	
	local finalx=finalmultiplier*adjustedx
	local finaly=finalmultiplier*adjustedy
	local finalz=finalmultiplier*adjustedz

	
	--self.frame:UseModelCenterToTransform(true)	--TODO: add model center to modelinfo
	self.frame:SetPosition(finalx,finaly,finalz)
	
	-- if(self.owner.IsPlayerControlled and self.owner:IsPlayerControlled())then
		-- print("Camera:")
		-- print(self.frame:GetCameraPosition())
	-- end
	
	-- if(self.owner.currentfacing_3D)then
	if(self.owner.currentfacing)then
		-- self.frame:SetFacing(self.owner.currentfacing_3D)
		local facing=self:GetFacingRadians()
		self.frame:SetFacing(facing)
	end
	

	
	----already did this earlier!
	-- self.frame:SetCameraPosition(cposz,-cposx,cposy)
	-- self.frame:SetCameraTarget(clookatz,-clookatx,clookaty)
	



	
	--self.frame:UseModelCenterToTransform(true)
	--local XXX=0
	--local YYY=20
	-- self.frame:SetCameraPosition(-10+XXX,YYY,5)
	-- self.frame:SetCameraTarget(0+XXX,YYY,0)	
	-- self.frame:SetPosition(0+XXX,YYY*1.5,0)
	
	--DevTools_Dump(self.frame:GetViewTranslation())
	self:CheckLighting()
	self:CheckSequenceMode()
end


function TRAININGGROUNDS_Drawable:Get3DPositionCoords()
	--override as necessary, don't call super
	return self.x,self.y,self.z
end

function TRAININGGROUNDS_Drawable:GetFacingRadians()
	--override as necessary, don't call super
	return self.owner.currentfacing
end


function TRAININGGROUNDS_Drawable:SetFrameSizeAndOrientation(wid,hgt,yaw,pitch)
	--override; don't call super when overriding
	self.frame:SetSize(wid,hgt)
end




function TRAININGGROUNDS_Drawable:TryDirectedSpellcast()
	self:SetAnimation(TRAININGGROUNDS_ANIMATION_LIST.SPELLCAST_DIRECTED)
end
--TODO: clean up names -- "complete" vs "finish"
function TRAININGGROUNDS_Drawable:TryCompleteDirectedSpellcast()
	self:StopSpellcast()
	self:SetAnimationAndRestart(TRAININGGROUNDS_ANIMATION_LIST.SPELLCAST_DIRECTED_FINISH)
end

function TRAININGGROUNDS_Drawable:StopSpellcast()
	self.animationsystem:ClearAnimationByCategory(TRAININGGROUNDS_ANIMATION_PRIORITY.SPELLCASTING)
end

--TODO: TryAttackMelee function which checks character's equipped weapon

function TRAININGGROUNDS_Drawable:TryAttackUnarmed()
	self:SetAnimationAndRestart(TRAININGGROUNDS_ANIMATION_LIST.Attack2H)
end

function TRAININGGROUNDS_Drawable:TryAttack2H()
	self:SetAnimationAndRestart(TRAININGGROUNDS_ANIMATION_LIST.Attack2H)
end

function TRAININGGROUNDS_Drawable:TryAttack2HL()
	self:SetAnimationAndRestart(TRAININGGROUNDS_ANIMATION_LIST.Attack2HL)
end

function TRAININGGROUNDS_Drawable:TryTaunt()
	self:SetAnimationAndRestart(TRAININGGROUNDS_ANIMATION_LIST.TAUNT)
end

function TRAININGGROUNDS_Drawable:Step(elapsed)
	--TODO: empty virtual function, maybe
end

function TRAININGGROUNDS_Drawable:PreDraw(elapsed)
	if(self.animationsystem)then
		self.animationsystem:PreDraw(elapsed)
	end
end

function TRAININGGROUNDS_Drawable:Cleanup()
	if(self.frames)then
		for i=1,#self.frames do
			self.frames[i]:Cleanup()
		end
	else
		self.frame:Cleanup()
	end
end

function TRAININGGROUNDS_Drawable:SetAnimation(sequence)
	self.animationsystem:SetAnimation(sequence)
end

function TRAININGGROUNDS_Drawable:SetAnimationAndRestart(sequence)
	self.animationsystem:SetAnimationAndRestart(sequence)
end

function TRAININGGROUNDS_Drawable:SetAnimationByNumber(index,variation,starttime,timescale)
	--print("Drawable:SetAnimationByNumber",index)
	--override
	--TODO: we recently copied the following from Drawable_CharacterModel.
		-- determine whether we can make it baseline without breaking sprites
	if(self.frame and self.frame.SetAnimation)then
		if(not starttime)then
			self.frame:SetAnimation(index,variation)
			self.sequencemode=false
		else
			-- SetSequenceTime runs on milliseconds, even if the rest of the game doesn't
			self.frame:SetSequenceTime(index,starttime*1000)
			self.sequencemode=true
			self.sequencestarttime=starttime
			self.sequenceplaybackstarttime=GetTime()
			if(timescale==nil)then timescale=1.0 end
			self.sequencetimescale=timescale
		end
	end
end

function TRAININGGROUNDS_Drawable:SetRawAnimation(index,variation,starttime,timescale)
	--override
end

function TRAININGGROUNDS_Drawable:CheckLighting()
	--override
end

function TRAININGGROUNDS_Drawable:CheckSequenceMode()
	--override
end
